function [F,T] = sparseOF(A,B,Cy,D,Cu,S,N)
% A, B, Cy, D defines T_{yw}(C)
% A, B, Cu, 0 defines T_{uw}(C)
% S: the sparse pattern
% N: the memory length of FIR
% F: decentralized controller
% T: the plant [T_{uw}(C)-C_s T_{yw}(C)] in objective function

disp('Sparse filter calculating...');

Q_out = size(Cu,1);
Q_in = size(Cy,1);
Q_r = N+1;

% form the state space realization of decentralized controller
Bq = [zeros(Q_in*(Q_r-2),Q_in);eye(Q_in)];
Aq = [zeros(Q_in*(Q_r-2),Q_in),eye(Q_in*(Q_r-2))];
Aq = [Aq;zeros(Q_in,Q_in*(Q_r-1))];

[m,n] = size(A);
[mq,nq] = size(Aq);

Psize = m*2+mq;

% solve optimization in step (4) using CVX
cvx_begin sdp quiet
cvx_solver mosek;
variable Q(Q_out,Q_in,Q_r);
variable r(1);
variable P(Psize,Psize) semidefinite;
expression Cq;
expression Dq;

Dq = Q(:,:,1);
Cq = Q(:,:,2);
for i = 3:Q_r
    Cq = [Q(:,:,i), Cq];
end

% state space realization of [T_{uw}(C)-C_s T_{yw}(C)]
Am = [A,zeros(m,n+nq);...
    zeros(m,n),A,zeros(m,nq);...
    zeros(mq,n),Bq*Cy,Aq];
Bm = [B;B;Bq*D];
Cm = [Cu,-Dq*Cy,-Cq];
Dm = -Dq*D;

minimize r
subject to
% sparsity constraints
if ~all(all(S))
    [I,J] = ind2sub(size(S),find(S==0));
    for k = 1:length(I)
        i = I(k);
        j = J(k);
        Q(i,j,:) == 0;
    end
end
% the below condition comes from Gahinet, Pascal, and Pierre Apkarian. "A 
% linear matrix inequality approach to H? control." International journal 
% of robust and nonlinear control 4.4 (1994): 421-448.
-[-P,P*Am,P*Bm,zeros(Psize,size(Cm,1)); ...
    Am'*P,-P,zeros(Psize,size(Bm,2)),Cm'; ...
    Bm'*P,zeros(size(Bm,2),Psize),-r*eye(size(Bm,2)),Dm'; ...
    zeros(size(Cm,1),Psize),Cm,Dm,-r*eye(size(Cm,1))]>=0;
cvx_end

% check the precision of CVX solution
fprintf('Status: %s\t/\tOptimal value: %0.4f\n',cvx_status,cvx_optval);
disp('Check the solutions...');
fprintf('Objective function value is %0.4f\n',r);
eigval = eig(full(-[-P,P*Am,P*Bm,zeros(Psize,size(Cm,1)); ...
    Am'*P,-P,zeros(Psize,size(Bm,2)),Cm'; ...
    Bm'*P,zeros(size(Bm,2),Psize),-r*eye(size(Bm,2)),Dm'; ...
    zeros(size(Cm,1),Psize),Cm,Dm,-r*eye(size(Cm,1))]));
fprintf('Minimum eigenvalue of LMI constraint: %0.4f\n',min(eigval));

Qnorm = S*0;
for i = 1:size(S,1)
    for j = 1:size(S,2)
        Qnorm(i,j) = norm(reshape(Q(i,j,:),[],1));
    end
end
fprintf('Sparse pattern is satisfied to precision: %0.4f\n',norm(Qnorm.*double(~S),'fro'));

% form the decentralized controller and corresponding plant [T_{uw}(C)-C_s T_{yw}(C)]
F = ss(Aq,Bq,Cq,Dq,-1);
Am = full(Am); Bm = full(Bm); Cm = full(Cm); Dm = full(Dm);
T = ss(Am,Bm,Cm,Dm,-1);